<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Defines a confirmation form to release an escrow payment.
 */
class ConfirmEscrowCancel extends ConfirmFormBase {

  /**
   * ID of the transaction.
   *
   * @var int
   */
  protected $transaction_id;

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "confirm_escrow_cancel";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $transaction_id = NULL) {
    // Check if the user is allowed to perform the operation.
    $is_allowed = _ubercart_funds_escrow_validation($transaction_id, 'cancel');

    if ($is_allowed) {
      $this->transaction_id = $transaction_id;

      return parent::buildForm($form, $form_state);
    }
    else {
      throw new AccessDeniedHttpException();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('view.uc_funds_user_transactions.incoming_escrow_payments');
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to cancel that escrow payment?');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $transaction_id = NULL) {
    $transaction_id = $this->transaction_id;
    $current_user_uid = \Drupal::currentUser()->id();

    // Load the transaction.
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_transactions WHERE transaction_id = :id', [
      ':id' => $transaction_id,
    ]);
    $transaction = $query->fetchObject();

    // Get issuer balance.
    $issuer_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $transaction->issuer,
    ])->fetchObject();

    // Update issuer balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => $transaction->issuer])
      ->fields([
        'uid' => $transaction->issuer,
        'balance' => $issuer_balance->balance += $transaction->net_amount,
      ])
      ->execute();

    // Update transaction.
    $connexion->merge('uc_funds_transactions')
      ->key(['transaction_id' => $transaction_id])
      ->fields([
        'status' => 'Cancelled',
      ])
      ->execute();

    // Load necessary parameters for email.
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');
    $issuer = User::load($transaction->issuer);
    $recipient = User::load($transaction->recipient);

    // The user who canceled the escrow is the issuer.
    if ($current_user_uid == $transaction->issuer) {
      // Send an HTML email to the recipient of the escrow payment.
      $mailManager = \Drupal::service('plugin.manager.mail');
      $langcode = \Drupal::config('system.site')->get('langcode');
      $params = [
        'subject' => $this->t('Escrow payment cancelled'),
        'title' => $this->t('Dear @recipient,', [
          '@recipient' => $recipient->getAccountName(),
        ]),
        'body' => $this->t('@issuer has cancelled an escrow payment of @amount to your account.', [
          '@issuer' => $issuer->getAccountName(),
          '@amount' => uc_currency_format($transaction->brut_amount / 100),
        ]),
      ];
      $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $recipient->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

      // Set a confirmation message to issuer.
      drupal_set_message($this->t('Escrow payment of @amount to @user has been cancelled. @user has been notified by email.', [
        '@amount' => uc_currency_format($transaction->brut_amount / 100),
        '@user' => $recipient->getAccountName(),
      ]), 'status');
    }

    // The user who canceled the escrow is the recipient.
    if ($current_user_uid == $transaction->recipient) {
      // Send an HTML email to the recipient of the escrow payment.
      $mailManager = \Drupal::service('plugin.manager.mail');
      $langcode = \Drupal::config('system.site')->get('langcode');
      $params = [
        'subject' => $this->t('Escrow payment cancelled'),
        'title' => $this->t('Dear @issuer,', [
          '@issuer' => $recipient->getAccountName(),
        ]),
        'body' => $this->t('@recipient has cancelled an escrow payment of @amount from your account.', [
          '@recipient' => $recipient->getAccountName(),
          '@amount' => uc_currency_format($transaction->brut_amount / 100),
        ]),
      ];
      $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $issuer->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

      // Set a confirmation message to recipient.
      drupal_set_message($this->t('Escrow payment of @amount from @user has been cancelled. @user has been notified by email.', [
        '@amount' => uc_currency_format($transaction->brut_amount / 100),
        '@user' => $issuer->getAccountName(),
      ]), 'status');
    }

    // Set redirection.
    $form_state->setRedirect('view.uc_funds_user_transactions.incoming_escrow_payments');
  }

}
