<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;

/**
 * Defines a confirmation form to release an escrow payment.
 */
class ConfirmEscrowRelease extends ConfirmFormBase {

  /**
   * ID of the transaction.
   *
   * @var int
   */
  protected $transaction_id;

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "confirm_escrow_release";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $transaction_id = NULL) {
    // Check if the user is allowed to perform the operation.
    $is_allowed = _ubercart_funds_escrow_validation($transaction_id, 'release');

    if ($is_allowed) {
      $this->transaction_id = $transaction_id;

      return parent::buildForm($form, $form_state);
    }
    else {
      throw new AccessDeniedHttpException();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('view.uc_funds_user_transactions.incoming_escrow_payments');
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to release that escrow payment?');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $transaction_id = NULL) {
    $transaction_id = $this->transaction_id;

    // Load the transaction.
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_transactions WHERE transaction_id = :id', [
      ':id' => $transaction_id,
    ]);
    $transaction = $query->fetchObject();

    // Check if recipient already have a balance.
    $recipient_balance_exist = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $transaction->recipient,
    ])->fetchObject();

    // Set the recipient balance to 0 if doesn't exist.
    $recipient_balance = $recipient_balance_exist ? $recipient_balance_exist->balance : 0;

    // Write recipient balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->insertFields([
        'uid' => $transaction->recipient,
        'balance' => $transaction->brut_amount,
      ])
      ->updateFields([
        'balance' => $recipient_balance += $transaction->brut_amount,
      ])
      ->key(['uid' => $transaction->recipient])
      ->execute();

    // Update site balance.
    $site_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => 1,
    ])->fetchAssoc();
    $site_balance['balance'] += $transaction->fee;
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => 1])
      ->fields($site_balance)
      ->execute();

    // Update transaction.
    $connexion->merge('uc_funds_transactions')
      ->key(['transaction_id' => $transaction_id])
      ->fields([
        'status' => 'Released',
      ])
      ->execute();

    // Load necessary parameters for email.
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');
    $issuer = User::load($transaction->issuer);
    $recipient = User::load($transaction->recipient);

    // Send an HTML email to the recipient of the escrow payment.
    $mailManager = \Drupal::service('plugin.manager.mail');
    $langcode = \Drupal::config('system.site')->get('langcode');
    $params = [
      'subject' => $this->t('Escrow payment released'),
      'title' => $this->t('Dear @recipient,', [
        '@recipient' => $recipient->getAccountName(),
      ]),
      'body' => $this->t('@issuer has released an escrow payment of @amount to your account. Your balance has been updated accordingly.', [
        '@issuer' => $issuer->getAccountName(),
        '@amount' => uc_currency_format($transaction->brut_amount / 100),
      ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $recipient->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // Set a confirmation message to user.
    if (!$transaction->fee) {
      drupal_set_message($this->t('You have transfered @amount @currency to @user.', [
        '@amount' => $transaction->brut_amount / 100,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
      ]), 'status');
    }
    if ($transaction->fee) {
      drupal_set_message($this->t('You have transfered @amount @currency to @user with a commission of %commission @currency.', [
        '@amount' => $transaction->brut_amount / 100,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
        '%commission' => $transaction->fee / 100,
      ]), 'status');
    }

    // Set redirection.
    $form_state->setRedirect('view.uc_funds_user_transactions.incoming_escrow_payments');
  }

}
