<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * Defines a confirmation form to approve a withdrawal request.
 */
class ConfirmWithdrawalApproval extends ConfirmFormBase {

  /**
   * ID of the withdrawal request.
   *
   * @var int
   */
  protected $request_id;

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "confirm_withdrawal_approval";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $request_id = NULL) {
    $this->request_id = $request_id;
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('view.uc_funds_withdrawal_requests.all_withdrawal_requests');
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to approve request: %id?', ['%id' => $this->request_id]);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $request_id = NULL) {
    $request_id = $this->request_id;

    // Load the request.
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_withdrawal_requests WHERE request_id = :id', [
      ':id' => $request_id,
    ]);
    $request = $query->fetchAssoc();

    // Update account balance.
    $requester_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $request['issuer'],
    ])->fetchObject();
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => $request['issuer']])
      ->fields([
        'uid' => $request['issuer'],
        'balance' => $requester_balance->balance -= $request['net_amount'],
      ])
      ->execute();

    // Update site balance.
    $site_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => 1,
    ])->fetchAssoc();
    $site_balance['balance'] += $request['fee'];
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => 1])
      ->fields($site_balance)
      ->execute();

    // Update request.
    $request['status'] = 'Approved';
    $connexion->merge('uc_funds_withdrawal_requests')
      ->key(['request_id' => $request_id])
      ->fields($request)
      ->execute();

    // Send an email to the requester.
    $requester = User::load($request['issuer']);
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    $mailManager = \Drupal::service('plugin.manager.mail');
    $langcode = \Drupal::config('system.site')->get('langcode');

    $params = [
      'subject' => $this->t('Withdrawal request validated'),
      'title' => $this->t('Dear @requester,', [
        '@requester' => $requester->getAccountName(),
      ]),
      'body' => $this->t('A site administrator has just validated your withdrawal request of @amount (@currency).<br>
     The money will be credited on your @method account in due order', [
       '@issuer' => $requester->getAccountName(),
       '@amount' => uc_currency_format($request['brut_amount'] / 100),
       '@currency' => $currency,
       '@method' => $request['method'],
     ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $requester->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // Confirmation message.
    drupal_set_message($this->t('Request approved. A confirmation email has been sent to @user', [
      '@user' => $requester->getAccountName(),
    ]), 'status');

    // Set redirection.
    $form_state->setRedirect('view.uc_funds_withdrawal_requests.all_withdrawal_requests');
  }

}
