<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\user\Entity\User;

/**
 * Defines a confirmation form to decline a withdrawal request.
 */
class ConfirmWithdrawalDecline extends ConfirmFormBase {

  /**
   * ID of the withdrawal request.
   *
   * @var int
   */
  protected $request_id;

  /**
   * {@inheritdoc}
   */
  public function getFormId() : string {
    return "confirm_withdrawal_decline";
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, string $request_id = NULL) {
    $this->request_id = $request_id;

    $form['reason'] = [
      '#type' => 'textarea',
      '#title' => t('Reason for decline'),
      '#description' => $this->t('The message will be addressed to the requester by email.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('view.uc_funds_withdrawal_requests.all_withdrawal_requests');
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return t('Are you sure you want to decline request: %id?', ['%id' => $this->request_id]);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state, string $request_id = NULL) {
    $request_id = $this->request_id;

    // Load the request.
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_withdrawal_requests WHERE request_id = :id', [
      ':id' => $request_id,
    ]);
    $request = $query->fetchAssoc();

    // Update request.
    $request['status'] = 'Declined';
    $request['reason'] = $form_state->getValue('reason');
    $connexion->merge('uc_funds_withdrawal_requests')
      ->key(['request_id' => $request_id])
      ->fields($request)
      ->execute();

    // Send an email to the requester.
    $requester = User::load($request['issuer']);
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    $mailManager = \Drupal::service('plugin.manager.mail');
    $langcode = \Drupal::config('system.site')->get('langcode');

    $params = [
      'subject' => $this->t('Withdrawal request declined'),
      'title' => $this->t('Dear @requester,', [
        '@requester' => $requester->getAccountName(),
      ]),
      'body' => $this->t('A site administrator has just declined your withdrawal request of @amount (@currency).<br>
     Reason:<br>@reason<p></p>', [
       '@issuer' => $requester->getAccountName(),
       '@amount' => uc_currency_format($request['brut_amount'] / 100),
       '@currency' => $currency,
       '@reason' => !empty($request['reason']) ? $request['reason'] : $this->t('No reason have been indicated.'),
     ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $requester->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // Confirmation message.
    drupal_set_message($this->t('Request declined. An email with the reason has been sent to @user', [
      '@user' => $requester->getAccountName(),
    ]), 'status');

    // Set redirection.
    $form_state->setRedirect('view.uc_funds_withdrawal_requests.all_withdrawal_requests');
  }

}
