<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;

/**
 * Form to transfer create an escrow to another user account.
 */
class UserFundsEscrow extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'uc_funds_escrow_payment';
  }

  /**
   * {@inheritdoc}
   *
   * Https://www.drupal.org/docs/8/api/form-api/configformbase-with-simple-configuration-api.
   */
  protected function getEditableConfigNames() {
    return [
      'uc_funds.escrow',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('uc_funds.escrow');
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');
    $fees_description = _ubercart_funds_print_transaction_fees('escrow');

    $form['amount'] = [
      '#type' => 'number',
      '#min' => 0.0,
      '#title' => $this->t('Amount to put in Escrow (@currency)',
        ['@currency' => $currency]
      ),
      '#description' => $fees_description,
      '#step' => 0.01,
      '#default_value' => 0.0,
      '#size' => 30,
      '#maxlength' => 128,
      '#required' => TRUE,
    ];

    $form['username'] = [
      '#id' => 'uc-funds-escrow-to',
      '#title' => $this->t('Allocated To'),
      '#description' => $this->t('Please enter the username of the user you want to allocate the escrow to.'),
      '#type' => 'entity_autocomplete',
      '#target_type' => 'user',
      '#required' => TRUE,
      '#size' => 30,
      '#maxlength' => 128,
      '#selection_settings' => [
        'include_anonymous' => FALSE,
      ],
    ];

    $form['notes'] = [
      '#type' => 'textarea',
      '#title' => t('Notes'),
      '#description' => t('Eventually add a message to the recipient.'),
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create escrow'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Get the commission applied.
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('escrow', $amount);

    // Get user balance.
    $account = \Drupal::currentUser();
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_user_funds WHERE uid = :uid', [
      ':uid' => $account->id(),
    ]);
    $user_balance = $query->fetchObject();

    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    // Error if the user doesn't have enought money to cover the escrow + fee.
    if ($user_balance->balance < intval($fee_applied['transaction_after_fee'])) {
      $form_state->setErrorByName('amount', $this->t("You don't have enough funds to cover this escrow.<br>
      The commission applied is %commission (@currency).",
      [
        '%commission' => $fee_applied['fee'] / 100,
        '@currency' => $currency,
      ]));
    }

    // Error if user try to make an escrow to itself.
    $recipient = User::load($form_state->getValue('username'));
    if ($account->id() == $recipient->id()) {
      $form_state->setErrorByName('username', $this->t("You are trying to make an escrow payment to yourself."));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('escrow', $amount);

    $connexion = \Drupal::database();
    $issuer = \Drupal::currentUser();
    $recipient = User::load($form_state->getValue('username'));

    $transaction = [
      'issuer' => $issuer->id(),
      'type' => 'Escrow',
      'recipient' => $recipient->id(),
      'created' => time(),
      'brut_amount' => $amount * 100,
      'net_amount' => $fee_applied['transaction_after_fee'],
      'fee' => $fee_applied['fee'],
      'status' => 'Pending',
      'notes' => $form_state->getValue('notes'),
    ];

    // Get issuer balance and apply transaction cost.
    $issuer_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $issuer->id(),
    ])->fetchObject();
    $issuer_balance->balance -= $fee_applied['transaction_after_fee'];

    // Update issuer balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => $issuer->id()])
      ->fields([
        'uid' => $issuer->id(),
        'balance' => $issuer_balance->balance,
      ])
      ->execute();

    // Write transaction in db.
    $connexion->insert('uc_funds_transactions')
      ->fields($transaction)
      ->execute();

    // Load currency code.
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    // Send an HTML email to the recipient.
    // @TODO Create a rule to handle the messages,
    // when rules will have Typed data implemented.
    $mailManager = \Drupal::service('plugin.manager.mail');
    $langcode = \Drupal::config('system.site')->get('langcode');

    $params = [
      'subject' => $this->t('New escrow payment!'),
      'title' => $this->t('Dear @recipient,', [
        '@recipient' => $recipient->getAccountName(),
      ]),
      'body' => $this->t('@issuer just created an escrow payment of @amount @currency which you are destinary of.', [
        '@issuer' => $issuer->getAccountName(),
        '@amount' => $amount,
        '@currency' => $currency,
      ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $recipient->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // Set a confirmation message to user.
    if (!$fee_applied['fee']) {
      drupal_set_message(t('Escrow payment of @amount @currency successfully created to @user.', [
        '@amount' => $amount,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
      ]), 'status');
    }
    if ($fee_applied['fee']) {
      drupal_set_message(t('Escrow payment of @amount @currency successfully created to @user with a commission of %commission @currency.',
      [
        '@amount' => $amount,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
        '%commission' => $fee_applied['fee'] / 100,
      ]), 'status');
    }
  }

}
