<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;

/**
 * Form to transfer money to another user account.
 */
class UserFundsTransfer extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'uc_funds_transfer_funds';
  }

  /**
   * {@inheritdoc}
   *
   * Https://www.drupal.org/docs/8/api/form-api/configformbase-with-simple-configuration-api.
   */
  protected function getEditableConfigNames() {
    return [
      'uc_funds.transfer',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('uc_funds.transfer');
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');
    $fees_description = _ubercart_funds_print_transaction_fees('transfer');

    $form['amount'] = [
      '#type' => 'number',
      '#min' => 0.0,
      '#title' => $this->t('Transfer Amount (@currency)',
        ['@currency' => $currency]
      ),
      '#description' => $fees_description,
      '#step' => 0.01,
      '#default_value' => 0.0,
      '#size' => 30,
      '#maxlength' => 128,
      '#required' => TRUE,
    ];

    $form['username'] = [
      '#id' => 'uc-funds-transfer-to',
      '#title' => $this->t('Transfer To'),
      '#description' => $this->t('Please enter the username of who you want to transfer the funds to.'),
      '#type' => 'entity_autocomplete',
      '#target_type' => 'user',
      '#required' => TRUE,
      '#size' => 30,
      '#maxlength' => 128,
      '#selection_settings' => [
        'include_anonymous' => FALSE,
      ],
    ];

    $form['notes'] = [
      '#type' => 'textarea',
      '#title' => t('Notes'),
      '#description' => t('Eventually add a message to the recipient.'),
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Transfer funds'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Get the commission applied.
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('transfer', $amount);

    // Get user balance.
    $account = \Drupal::currentUser();
    $connexion = \Drupal::database();
    $query = $connexion->query('SELECT * FROM uc_funds_user_funds WHERE uid = :uid', [
      ':uid' => $account->id(),
    ]);
    $user_balance = $query->fetchObject();

    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    // Error if the user doesn't have enough money to cover the transfer + fee.
    if ($user_balance->balance < intval($fee_applied['transaction_after_fee'])) {
      $form_state->setErrorByName('amount', $this->t("You don't have enough funds to cover this transfer.<br>
      The commission applied is %commission (@currency).",
      [
        '%commission' => $fee_applied['fee'] / 100,
        '@currency' => $currency,
      ]));
    }

    // Error if user try to send money to itself.
    $recipient = User::load($form_state->getValue('username'));
    if ($account->id() == $recipient->id()) {
      $form_state->setErrorByName('username', $this->t("You are trying to transfer money to yourself."));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('transfer', $amount);

    $connexion = \Drupal::database();
    $issuer = \Drupal::currentUser();
    $recipient = User::load($form_state->getValue('username'));

    $transaction = [
      'issuer' => $issuer->id(),
      'type' => 'Transfer',
      'recipient' => $recipient->id(),
      'created' => time(),
      'brut_amount' => $amount * 100,
      'net_amount' => $fee_applied['transaction_after_fee'],
      'fee' => $fee_applied['fee'],
      'status' => 'Completed',
      'notes' => $form_state->getValue('notes'),
    ];

    // Get issuer balance.
    $issuer_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $issuer->id(),
    ])->fetchObject();

    // Check if recipient already have a balance.
    $recipient_balance_exist = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $recipient->id(),
    ])->fetchObject();

    // Set recipient balance to 0 if doesn't exist.
    $recipient_balance = $recipient_balance_exist ? $recipient_balance_exist->balance : 0;

    // Write recipient balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->insertFields([
        'uid' => $recipient->id(),
        'balance' => $transaction['brut_amount'],
      ])
      ->updateFields([
        'balance' => $recipient_balance += $transaction['brut_amount'],
      ])
      ->key(['uid' => $recipient->id()])
      ->execute();

    // Update issuer balance in db.
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => $issuer->id()])
      ->fields([
        'uid' => $issuer->id(),
        'balance' => $issuer_balance->balance -= $fee_applied['transaction_after_fee'],
      ])
      ->execute();

    // Write transaction in db.
    $connexion->insert('uc_funds_transactions')
      ->fields($transaction)
      ->execute();

    // Update site balance.
    $site_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => 1,
    ])->fetchAssoc();
    $site_balance['balance'] += $fee_applied['fee'];
    $connexion->merge('uc_funds_user_funds')
      ->key(['uid' => 1])
      ->fields($site_balance)
      ->execute();

    // Load currency code.
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    // Send an HTML email to recipient and issuer.
    // @TODO Create a rule to handle the messages,
    // when rules will have Typed data implemented.
    $mailManager = \Drupal::service('plugin.manager.mail');
    $langcode = \Drupal::config('system.site')->get('langcode');
    // To recipient.
    $params = [
      'subject' => $this->t('New transfer!'),
      'title' => $this->t('Dear @recipient,', [
        '@recipient' => $recipient->getAccountName(),
      ]),
      'body' => $this->t('@issuer just transfered @amount (@currency) to your account.', [
        '@issuer' => $issuer->getAccountName(),
        '@amount' => uc_currency_format($amount),
        '@currency' => $currency,
      ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $recipient->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // To issuer.
    $params = [
      'subject' => $this->t('Transfer confirmed.'),
      'title' => $this->t('Dear @issuer,', [
        '@issuer' => $issuer->getAccountName(),
      ]),
      'body' => $this->t('Your transfer to @recipient of @amount (@currency) is confirmed and has been processed.', [
        '@recipient' => $recipient->getAccountName(),
        '@amount' => uc_currency_format($amount),
        '@currency' => $currency,
      ]),
    ];
    $mailManager->mail('ubercart_funds', 'uc_funds_transaction', $issuer->getEmail(), $langcode, $params, $reply = NULL, $send = TRUE);

    // Set a confirmation message to user.
    if (!$fee_applied['fee']) {
      drupal_set_message($this->t('You have transfered @amount @currency to @user', [
        '@amount' => $amount,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
      ]), 'status');
    }
    if ($fee_applied['fee']) {
      drupal_set_message($this->t('You have transfered @amount @currency to @user with a commission of %commission @currency', [
        '@amount' => $amount,
        '@currency' => $currency,
        '@user' => $recipient->getAccountName(),
        '%commission' => $fee_applied['fee'] / 100,
      ]), 'status');
    }
  }

}
