<?php

namespace Drupal\ubercart_funds\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form to withdraw mmoney on user account.
 */
class UserFundsWithdraw extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'uc_funds_withdraw';
  }

  /**
   * {@inheritdoc}
   *
   * Https://www.drupal.org/docs/8/api/form-api/configformbase-with-simple-configuration-api.
   */
  protected function getEditableConfigNames() {
    return [
      'uc_funds.withdraw',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('uc_funds.withdraw');
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    $methods = ubercart_funds_enabled_withdrawal_methods();
    foreach ($methods as $key => $method) {
      $fee = _ubercart_funds_print_method_fees('withdraw_' . $method['id'])->render();
      $readable_methods[$key] = $method['name']->render() . ' ' . $fee;
    }
    $form['amount'] = [
      '#type' => 'number',
      '#min' => 0,
      '#title' => $this->t('Amount to withdraw (@currency)', [
        '@currency' => $currency,
      ]
      ),
      '#description' => $this->t('Enter the amount you want to withdraw.'),
      '#default_value' => 0,
      '#size' => 30,
      '#maxlength' => 128,
      '#required' => TRUE,
    ];

    $form['methods'] = [
      '#type' => 'radios',
      '#options' => $readable_methods,
      '#title' => t('Select your prefered Withdrawal method'),
      '#default_value' => '',
      '#required' => TRUE,
    ];

    $form['actions'] = ['#type' => 'actions'];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit Request'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $method = $amount = $form_state->getValue('methods');;
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('withdraw_' . $method, $amount);
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    $connexion = \Drupal::database();
    $issuer = \Drupal::currentUser();
    $user_datas = \Drupal::service('user.data')->get('ubercart_funds', $issuer->id(), $method);

    // Get issuer balance.
    $issuer_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
      ':uid' => $issuer->id(),
    ])->fetchObject();

    if ($amount * 100 > $issuer_balance->balance) {
      $form_state->setErrorByName('amount', $this->t("Your available balance is @balance @currency.", [
        '@balance' => $issuer_balance->balance / 100,
        '@currency' => $currency,
      ]));
    }

    if ($fee_applied['transaction_after_fee'] > $issuer_balance->balance) {
      $form_state->setErrorByName('amount', $this->t('You cannot withdraw @amount @currency using this payment method. Commission is @fee @currency.', [
        '@amount' => $amount,
        '@currency' => $currency,
        '@fee' => $fee_applied['fee'] / 100,
      ]));
    }

    if (!$user_datas) {
      $form_state->setErrorByName('methods', $this->t('Please <a href="@enter_details_link">enter details</a> for this withdrawal method first.', [
        '@enter_details_link' => Url::fromRoute('uc_funds.withdrawal_methods')->toString() . '/' . $method,
      ]));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $method = $amount = $form_state->getValue('methods');;
    $amount = $form_state->getValue('amount');
    $fee_applied = _ubercart_funds_apply_fees('withdraw_' . $method, $amount);
    $currency = \Drupal::config('uc_store.settings')->get('currency.code');

    $connexion = \Drupal::database();
    $issuer = \Drupal::currentUser();

    $request = [
      'issuer' => $issuer->id(),
      'created' => time(),
      'method' => $method,
      'brut_amount' => $amount * 100,
      'net_amount' => $fee_applied['transaction_after_fee'],
      'fee' => $fee_applied['fee'],
      'status' => 'Pending approval',
    ];

    // Write withdrawal request in db.
    $connexion->insert('uc_funds_withdrawal_requests')
      ->fields($request)
      ->execute();

    // Set a confirmation message to user.
    drupal_set_message(t('Your Withdrawal Request has been sent and will be processed in due order.'));
    if ($fee_applied['fee']) {
      drupal_set_message(t('An extra commission of @fee @currency will be apllied to your withraw.', [
        '@fee' => $fee_applied['fee'] / 100,
        '@currency' => $currency,
      ]), 'status');
    }
  }

}
