<?php

namespace Drupal\ubercart_funds\Plugin\RulesAction;

use Drupal\uc_order\OrderInterface;
use Drupal\rules\Core\RulesActionBase;

/**
 * Add funds to balance rules action.
 *
 * @RulesAction(
 *   id = "uc_funds_add_funds_to_balance",
 *   label = @Translation("Add funds to balance"),
 *   category = @Translation("Ubercart funds"),
 *   context = {
 *     "entity" = @ContextDefinition("entity:uc_order",
 *       label = @Translation("Order"),
 *       description = @Translation("The deposit order which is updated")
 *     )
 *   }
 * )
 */
class AddFundsToBalance extends RulesActionBase {

  /**
   * Update account balance.
   *
   * @param \Drupal\uc_order\OrderInterface $order
   *   The order update triggered by rules.
   */
  protected function doExecute(OrderInterface $order) {
    // Clean the array as by the deposit form, products always come single.
    $product = reset($order->products);
    // Check if it's a deposit and if the order is paid.
    $is_deposit = preg_match('/^deposit_[0-9]*/', $product->get('model')->getValue()[0]['value']);
    $is_order_paid = strpos($order->getStatusId(), 'pending');

    if ($is_deposit && $is_order_paid) {
      // Get requested amount.
      $uid = $order->getOwnerId();
      $amount = $order->getTotal();
      $payment_method = $order->getPaymentMethodId();
      $connexion = \Drupal::database();

      $fee_applied = _ubercart_funds_apply_fees('deposit_' . $payment_method, $amount);
      // Override transaction after fee as it is substracted here.
      $fee_applied['transaction_after_fee'] = ($amount * 100) - $fee_applied['fee'];

      $transaction = [
        'issuer' => $uid,
        'recipient' => $uid,
        'type' => 'Deposit',
        'created' => time(),
        'brut_amount' => $amount * 100,
        'net_amount' => $fee_applied['transaction_after_fee'],
        'fee' => $fee_applied['fee'],
        'status' => 'Completed',
        'notes' => $this->t('Deposit of @amount (@currency)', [
          '@amount' => $fee_applied['transaction_after_fee'] / 100,
          '@currency' => $order->getCurrency(),
        ]),
      ];

      // Update account balance.
      $account_exist = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
        ':uid' => $uid,
      ])->fetchObject();

      $connexion->merge('uc_funds_user_funds')
        ->insertFields([
          'uid' => $uid,
          'balance' => $fee_applied['transaction_after_fee'],
        ])
        ->updateFields([
          'balance' => $fee_applied['transaction_after_fee'] += $account_exist->balance,
        ])
        ->key(['uid' => $uid])
        ->execute();

      // Update site balance.
      $site_balance = $connexion->query("SELECT * FROM uc_funds_user_funds WHERE uid = :uid", [
        ':uid' => 1,
      ])->fetchObject();
      $connexion->merge('uc_funds_user_funds')
        ->key(['uid' => 1])
        ->updateFields([
          'balance' => $site_balance->balance += $fee_applied['fee'],
        ])
        ->execute();

      // Write transaction in db.
      $connexion->insert('uc_funds_transactions')
        ->fields($transaction)
        ->execute();
    }
  }

}
